// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>

double min_excite_error=1e-9;


void calculate_kd(struct device *in)
{
int i;
double E;
double epc;
double mu_min;
double langevin;
//double Rbi;
double steps;
double start;
double stop;
double dx;
double x;
double tot;
double n;
double p;
double ni;
double a;
double kf;
double Eb;
double bo;
double Tlc;
double kd;
double div=0.0;
double yl;
double yr;
double phil;
double phic;
double phir;
FILE *out;
for (i=0;i<in->ymeshpoints;i++)
{

	if (i==0)
	{
		phil=in->Vapplied;
		yl=in->ymesh[0]-(in->ymesh[1]-in->ymesh[0]);
	}else
	{
		phil=in->phi[i-1];
		yl=in->ymesh[i-1];
	}

	if (i==(in->ymeshpoints-1))
	{
		phir=in->Vr;
		yr=in->ymesh[i]+(in->ymesh[i]-in->ymesh[i-1]);
	}else
	{
		phir=in->phi[i+1];
		yr=in->ymesh[i+1];
	}

	phic=in->phi[i];
	//printf("%e %e %e\n",phil,phic,phir);
	//getchar();
	double delta=yr-yl;
	n=in->n[i][0];
	p=in->p[i][0];
	ni=in->ni[i];
	a=in->a[i];
	kf=in->kf[i];
	Tlc=in->Tl[i];

	epc=in->epsilonr[i]*epsilon0;
	E=-(phir-phil)/delta;
	E=E;
	mu_min=in->mun[i][0];
	if (mu_min>in->mup[i][0]) mu_min=in->mup[i][0];
	//mu_min=(in->mup[i]+in->mun[i])/2.0;
	langevin=q*(mu_min)/(epc);

	steps=50.0;
	start=0.0;
	stop=a*0.1;

	dx=(stop-start)/steps;
	x=start+dx;
	tot=0.0;
//#define kddebug
#ifdef kddebug
FILE *out=fopen("./gaus.dat","w");
#endif
x=a*1.2;
	//do
	//{

		Eb=q*q/(4.0*pi*epc*x);
		bo=q*q*q*E/(8.0*pi*epc*kb*kb*Tlc*Tlc);
		kd=langevin*(exp(-Eb/(Tlc*kb))*(1.0+bo+bo*bo/3.0+bo*bo*bo/18.0+bo*bo*bo*bo/180))/(4.0*pi*x*x*x);
		div=kd/(kd+kf);
		//f=(4.0/(sqrt(pi)*a*a*a))*x*x*exp(-x*x/(a*a));
		tot=kd;
		//tot+=kd*f*dx;
		#ifdef kddebug
		fprintf(out,"%e %e %e %e\n",x,kd,f,kd*f);
		#endif

		//x+=dx;
	//}while(x<stop);
	in->kd[i]=tot*1.0;
	in->kr[i]=langevin/1000.0;

#ifdef kddebug
fclose(out);
getchar();
#endif

}


out=fopen("./exdump.dat","w");

for (i=0;i<in->ymeshpoints;i++)
{
	fprintf(out,"%e %e %e\n",in->ymesh[i],in->kd[i],in->phi[i]);
}

fclose(out);

//getchar();

}


double get_excite_error(struct device *in,double *b)
{
double tot=0.0;
int i;
for (i=0;i<in->ymeshpoints;i++)
{

	tot+=mod(b[i]);
}
return tot/1e15;
}


int solve_excite(struct device *in)
{
printf("Solve Excite\n");
calculate_kd(in);
int i;

int N=in->ymeshpoints*3-2;


int M=in->ymeshpoints;
int *Ti =    malloc(N*sizeof(int));
int *Tj =    malloc(N*sizeof(int));
double *Tx = malloc(N*sizeof(double));
double *b = malloc(M*sizeof(double));


double exl;
double exr;
double yl;
double yc;
double yr;
double dyl;
double dyr;
double dyc=0.0;
int ittr=0;
int pos=0;
double error;
double Del=0.0;
double Dec=0.0;
double Der=0.0;
double De0=0.0;
double De1=0.0;
double E=0.0;
double epc=0.0;
double Tlc;
double Rbi=0.0;
double nc;
double pc;
double ni;
double exc;
double dexl;
double dexc;
double dexr;
double bo;
double a;
double Eb;
double kf;
double kd;
double Rdis;
double Gex;

	do
	{

		pos=0;
		for (i=0;i<in->ymeshpoints;i++)
		{


			if (i==0)
			{
				exl=0.0;
				Del=in->Dex[0];
				//printf("Vl=%e\n",in->Vl);
				yl=in->ymesh[0]-(in->ymesh[1]-in->ymesh[0]);


			}else
			{
				Del=in->Dex[i-1];
				exl=in->ex[i-1];
				yl=in->ymesh[i-1];
			}

			if (i==(in->ymeshpoints-1))
			{
				exr=0.0;
				//printf("Vr=%e\n",in->Vr);
				Der=in->Dex[i];
				yr=in->ymesh[i]+(in->ymesh[i]-in->ymesh[i-1]);
			}else
			{
				Der=in->Dex[i+1];
				exr=in->ex[i+1];
				yr=in->ymesh[i+1];

			}

			exc=in->ex[i];
			Tlc=in->Tl[i];
			epc=in->epsilonr[i]*epsilon0;
			E=5e6;//(phir-phil)/(dyr+dyl);
			double mu_min=0.0;
			nc=in->n[i][0];
			pc=in->p[i][0];
			ni=in->ni[i];
			Gex=in->Gex[i];


			mu_min=(in->mup[i][0]+in->mun[i][0])/2.0;
			double langevin=q*(mu_min)/(epc);

			Rbi=langevin*(nc*pc-ni*ni);
			in->Rbi[i]=Rbi;

			bo=q*q*q*E/(8.0*pi*epc*kb*kb*Tlc*Tlc);
			a=in->a[i];
			Eb=q*q/(4.0*pi*epc*a);
			kf=in->kf[i];


			kd=in->kd[i];//3.0*Rbi*(exp(-Eb/(Tlc*kb))*(1.0+bo+bo*bo/3.0))/(4.0*pi*a*a*a);
			//printf("%e %e\n",kd,Rbi);
			Rdis=kd*exc;
			in->G[i]=Rdis;


			yc=in->ymesh[i];
			dyl=yc-yl;
			dyr=yr-yc;
			dyc=(dyl+dyr)/2.0;

			Dec=in->Dex[i];
			De0=(Del+Dec)/2.0;
			De1=(Dec+Der)/2.0;

			dexl=De0/dyl/dyc;
			dexc=-(De0/dyl/dyc+De1/dyr/dyc);
			dexr=De1/dyr/dyc;


			double dex=dexl*exl+dexc*exc+dexr*exr+Gex-kf*exc-kd*exc+Rbi;//
			dexc+=-kf-kd;

			if (i!=0)
			{
				Ti[pos]=i;
				Tj[pos]=i-1;
				Tx[pos]=dexl;
				pos++;



			}

			Ti[pos]=i;
			Tj[pos]=i;
			Tx[pos]=dexc;
			pos++;

			if (i!=(in->ymeshpoints-1))
			{
				Ti[pos]=i;
				Tj[pos]=i+1;
				Tx[pos]=dexr;
				pos++;

			}

			b[i]=-(dex);


		}

		solver(M,N,Ti,Tj, Tx,b);


		for (i=0;i<in->ymeshpoints;i++)
		{
			in->ex[i]+=b[i];///(1.0+fabs(b[i]/1e8/(300.0*kb/q)));
		}

		error=get_excite_error(in,b);
		printf("Excite error = %e\n",error);

	FILE *out;
	out =fopen ("./excite.dat","w");
	for (i=0;i<in->ymeshpoints;i++)
	{
			fprintf(out,"%e %e %e %e %e\n",in->ymesh[i],in->ex[i],in->n[i][0],in->p[i][0],in->n[i][0]*in->p[i][0]-in->ni[i]*in->ni[i]);
	}
	fclose(out);
	//plot_now_excite(in);
//getchar();
		ittr++;
	//getchar();
	if ((ittr<2)&&(error<min_excite_error)) in->excite_conv=TRUE;
	}while((ittr<20)&&(error>min_excite_error));




//exit(0);
free(Ti);
free(Tj);
free(Tx);
free(b);
return 0;
}

