// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>

double get_extracted_np(struct device *in)
{
int i;
int band;
int bands=in->bands;

double tot=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	for (band=0;band<bands;band++)
	{
		tot+=((in->n[i][band]-in->ninit[i])+(in->p[i][band]-in->pinit[i]))/2.0;
		//printf("%e %e\n",(in->n[i][band]-in->ninit[i]),in->p[i][band]-in->pinit[i]);
		//getchar();
	}
}


return tot/((double)(in->ymeshpoints));
}

double get_extracted_k(struct device *in)
{
int i;

double tot=0.0;
double n=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	tot+=in->Rbi[i];
	n+=in->n[i][0]*in->p[i][0];
}
//printf("%le %le %le\n",tot,n,tot/n);
//getchar();
return tot/n;
}

double get_avg_mue(struct device *in)
{
int i;
int band;
int bands=in->bands;

double tot=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	for (band=0;band<bands;band++)
	{
		tot+=in->mun[i][band];
	}
}
return tot/(((double)in->ymeshpoints*bands));
}

double get_avg_muh(struct device *in)
{
int i;
int band;
int bands=in->bands;

double tot=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	for (band=0;band<bands;band++)
	{
		tot+=in->mup[i][band];
	}
}
return tot/(((double)in->ymeshpoints*bands));
}


double get_total_n_charge(struct device *in)
{
int i;
int band;
int bands=in->bands;

double tot=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	for (band=0;band<bands;band++)
	{
		tot+=in->n[i][band];
	}
}

return tot;
}

double get_total_p_charge(struct device *in)
{
int i;
int band;
int bands=in->bands;

double tot=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	for (band=0;band<bands;band++)
	{
		tot+=in->p[i][band];
	}
}

return tot;
}

double get_total_n_trapped_charge(struct device *in)
{
int i;

double tot=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
		tot+=in->nt[i];
}

return tot;
}

double get_total_p_trapped_charge(struct device *in)
{
int i;

double tot=0.0;
for (i=0;i<in->ymeshpoints;i++)
{
	tot+=in->pt[i];
}

return tot;
}

